/*
 * ccupload.c - upload code to CC1 
 * 
 * Copyright (C) 2003 Chris Ruehl <ruehlc@wiesbaden.vistec.net>
 * original code-parts from Diman Todorov
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * ----------------------------------------------------------- 
 *  who | date | Comment
 * -----+------+----------------------------------------------
 */
#include <stdio.h>   /* Standard input/output definitions */
#include <stdlib.h>  /* atoi need it */
#include <unistd.h>  /* UNIX standard function definitions */
#include "serial.h"   /*prototypes*/

void usage ( char **called ) {
    	fprintf(stderr,"Usage: %s [options] <datfile> \n",
		called[0]);
    	fprintf(stderr," <datfile>  : The path and filename which to upload\n");
	fprintf(stderr,"Valid options are\n");
    	fprintf(stderr," -d <device>: Serial device (default /dev/ttyS1)\n");
	fprintf(stderr," -s : Upload System-bytes (default not send)\n");
	exit(1);
}

int main(int argc, char **argv)
{

#ifndef FALSE
#define FALSE 0
#define TRUE !FALSE
#endif
    int mainfd = 0;
    int opt;
    char opt_d=FALSE ;
    char opt_s=FALSE ;
    char *filename = NULL;
    char *devname = NULL;
    
    while(1)
    {        
        opt = getopt(argc, argv, "sd:");
	
	if(opt == -1) 
	    break;
	
        switch(opt)
	{
	    case 'd':
                opt_d=TRUE;
	    	devname = optarg;
	    break;  	  

	    case 's':
	    	/* -- do upload the system-bytes -- */
	    	opt_s=TRUE;
	    break;

	    default:
		usage(argv);
	    break;
        }
    }

	
    if ( optind == argc ) {
       fprintf(stderr,"DAT-File missing\n");
       usage(argv);
    }
    else {
	filename=argv[optind++];
    }

    if ( ! opt_d ) {
        devname="/dev/ttyS1";
    }
    
    file_avail(filename);

    if ( (mainfd = open_port(devname)) < 0 ) {
	usage(argv);
    }
    
    fprintf(stderr,"%s successfully open\n",devname);
    
    if(initmc(mainfd) < 0) {
	fprintf(stderr,"Error: failed mc init\n");
	close(mainfd);
	return 1;
    }
    else {
        fprintf(stderr,"intialzied micro controller successfully\n");
    }
    
    if(sendprog(filename, mainfd, opt_s) < 0) {
	fprintf(stderr,"failed programming\n");
	close(mainfd);
	return 1;
    }     
    else {
        fprintf(stderr,"programmed eeprom successfully\n");
    }
    close(mainfd);
    return 0;
}
